﻿#region

using System;
using System.Linq;
using JsMapper.BO.Attributes;
using JsMapper.BO.Config;
using JsMapper.BO.Events;

#endregion

namespace JsMapper.BL.Config
{
    public class JsModelConfigManager
    {
        private JsModelConfig _config;

        public JsModelConfig Config
        {
            get
            {
                if (_config == null) throw new NullReferenceException("Config must be initialised!");
                return _config;
            }
            set
            {
                _config = value;
                InitializeConfig();
            }
        }

        public void Configurate(JsModelConfig config)
        {
            Config = config;
        }

        public event JsModelCollectionHandler OnCollection;
        public event JsModelCacheHandler OnUseCache;

        public static bool IsConfigured { get; private set; }

        private void InitializeConfig()
        {
            if (Config.UseCache) UseCache();
            if (Config.ScanJsModels) ScanJsModels();

            IsConfigured = true;
        }

        private void ScanJsModels()
        {
            if (OnCollection == null) throw new NullReferenceException("No handlin for the scanned Models");

            var coll = AppDomain.CurrentDomain.GetAssemblies()
                                .SelectMany(assmbls => assmbls.GetTypes(), (a, t) => t)
                                .Where(x => Attribute.IsDefined(x, typeof (JsModelAttribute)))
                                .ToList();


            var collArgs = new JsModelCollectionEventArgs(coll);
            OnCollection(this, collArgs);
        }

        private void UseCache()
        {
            if (OnUseCache == null) return;

            var cacheArgs = new JsModelCacheEventArgs();
            OnUseCache(this, cacheArgs);
        }
    }
}